import jinro.*;
import scripts.*;
import scripts.roles.*;
import java.text.Collator;
import com.skype.*;

class Game extends System {
	def strings = super.strings.attachBundle("Game.properties");
	// Keeps roles/death secret from viewers if it's true.
	def issecret = false;

	// Returns the game options.
	// This method is called by JinroGame.getOptionList().
	def getOptionList() {
		def list = [];
		
		list << [strings.getString("option.issecret"), "issecret", "Boolean"];			//Secret mode
		
		return list;
	}
	// Returns the actors data (for displaying).
	// This method is called by JinroGame.getActorsData().
	def getActorsData() {
		def list = [];
		actors.each { actor ->
			def data = [];
			data[1] = actor.name;
			// Hides roles and death from viewers if the game is secret mode.
			if(issecret && actor.state.priority < 50 && actor.state != State.escaped) {
				data[0] = State.bitten.priority;
				data[3] = strings.getString("secret.death");
			} else {
				data[0] = actor.state.priority;
				data[3] = actor.state.name;
			}
			if(issecret) {
				data[2] = strings.getString("secret.role");
				data[4] = 0;
			} else {
				data[2] = JinroGame.getAlias(actor.getRoleName());
				data[4] = actor.getPriority();
			}
			list << data;
		};
		// Sort the actors.
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.setStrength(Collator.PRIMARY);
		list.sort { one, two ->
			if(one[0] != two[0]) {
				// Actor's state
				return two[0] - one[0];
			} else if(one[4] != two[4]) {
				// Actor's role
				return two[4] - one[4];
			} else {
				// Actor's name
				return collator.compare(one[1], two[1]);
			}
		};
		return list;
	}
	// This method is called by JinroInterface.Update.CountDown.run().
	// If it returns true , the tool notifies all actors of the remaining time.
	//               false, you should write a code of notifying here.
	def timerCounted(minutes, seconds) {
		return false;
	}
	// Frequency of revotes
	def revoteCount = 0;
	// Calls when start argument/night time.
	// This method is called by JinroGame.changePhase().
	def startPhase() {
	}
	// Calls when end argument/night time.
	// This method is called by JinroGame.changePhase().
	def endPhase() {
		return true;
	}
	// Checks if the game can go to next phase.
	// This method is called by JinroGame.checkStep().
	def checkStep() {
		return false;
	}
	def watchGeneralChat() {
		if(!monitor) {
			monitor = new ChatMessageMonitor();
		}
		monitor.addListener(listener);
	}
	def monitor;
	def listener;
	def GUI = [:];
	def asyncExec;
	def Game() {
		def regex = { "^" + it.replaceAll(/(\p{Punct})/, "\\\\\$1") + "\$" };
		def jistrings = JinroLocalization.getBundle("JinroInterface.properties");
		def minutes = regex(jistrings.getString("notice.timer.minutes", "%S")).replaceFirst(/\\%S/, "([0-9]+)");
		def seconds = regex(jistrings.getString("notice.timer.seconds", "%S")).replaceFirst(/\\%S/, "([0-9]+)");
		def executed = regex(strings.getString("notice.executed", "%S")).replaceFirst(/\\%S/, "(.+)");
		def bitten = regex(Civilian.strings.getString("notice.killed.normal", "%S")).replaceFirst(/\\%S/, "(.+)");
		def escaped = regex(Spy.strings.getString("notice.killed.escaped", "%S")).replaceFirst(/\\%S/, "(.+)");
		listener = [
			chatMessageReceived: { message ->
				if(!JinroSkype.allchat || JinroSkype.allchat?.id == message.chat?.id) {
					def min = message.content =~ minutes;
					if(min) {
						asyncExec { JinroInterface.setTimer((min[0][1] as int), 0) };
						JinroInterface.countdown = true;
					}
					def sec = message.content =~ seconds;
					if(sec) {
						asyncExec { JinroInterface.setTimer(0, (sec[0][1] as int)) };
						JinroInterface.countdown = true;
					}
					def exec = message.content =~ executed;
					if(exec) {
						def actor = getActor(exec[0][1]);
						if(actor?.isLive()) {
							actor.state = State.executed;
							asyncExec { JinroInterface.refreshUserList() };
						}
					}
					def bit = message.content =~ bitten;
					if(bit) {
						def actor = getActor(bit[0][1]);
						if(actor?.isLive()) {
							if(actor.roleIs(new Fox())) {
								actor.state = State.cursed;
							} else if(actor.roleIs(new Wolffortune())) {
								actor.state = State.recursed;
							} else if(actor.isWolf() || actor.roleIs(new Runaway())) {
								actor.state = State.followed;
							} else {
								actor.state = State.bitten;
							}
							asyncExec { JinroInterface.refreshUserList() };
						}
					}
					def esc = message.content =~ escaped;
					if(esc) {
						def actor = getActor(esc[0][1]);
						if(actor?.isLive()) {
							actor.state = State.escaped;
							asyncExec { JinroInterface.refreshUserList() };
						}
					}
				}
			},
			chatMessageSent: {}
		] as ChatMessageMonitor.Listener;
		def suffix = "_" + (JinroInterface.version =~ /-swing\b/ ? "Swing" : "SWT") + ".groovy";
		["Initialize"].each {
			GUI[it] = JinroGame.parseClass(new File(new File(JinroGame.getScriptsDirectory(), "scripts"), it + suffix));
		};
		def initialize = GUI.Initialize.newInstance();
		initialize.init();
		asyncExec = initialize.asyncExec;
	}
}
