package scripts;

import jinro.*;
import scripts.roles.*;
import java.text.Collator;

// Game class must inherit this class.

abstract class System {
	def strings = JinroLocalization.getBundle("System.properties");
	// List of Actors
	def actors = [];
	// First victim
	// This property is set by JinroGame.setFirstVictim(GroovyObject actor).
	def firstvictim;
	// Name of times
	// This property is referred by JinroGame.addLog(String message) etc.
	def timename = [strings.getString("time.daytime"), strings.getString("time.nighttime")];
	// Returns all state names.
	// This method is called by JinroGame.getStateNames().
	def getStateNames() {
		def list = [];
		State.each { list << it.name };
		return list;
	}
	// Returns the proper role class for the role's name.
	// This method is called by JinroGame.getRole(String name).
	def getRole(name) {
		return roles.find { it.newInstance().getRoleName() == name };
	}
	// Returns the proper state object for the state's name.
	// This method is called by JinroGame.getState(String name).
	def getState(name) {
		return State.find { it.name == name };
	}
	// Adds an actor.
	// This method is called by JinroGame.addActor(String name, String skypeid).
	def addActor(name, skypeid = null) {
		if(actors.find { it.name == name }) {
			return false;
		}
		Actor newactor = new Actor(name);
		if(skypeid) {
			if(actors.find { it.skypeid == skypeid }) {
				return false;
			}
			newactor.skypeid = skypeid;
		}
		actors << newactor;
		JinroGame.addLog(strings.getString("entry.added", newactor.name));
		return true;
	}
	// Removes an actor.
	// This method is called by JinroGame.removeActor(String name)
	def removeActor(name) {
		def actor = actors.find { it.name == name };
		actors.remove(actor);
		JinroGame.addLog(strings.getString("entry.removed", actor.name));
	}
	// Returns the proper actor object for the actor's name.
	// This method is called by JinroGame.getActor(String name)
	def getActor(name) {
		return actors.find { it.name == name };
	}
	// Checks if the name already exists as an actor's.
	// This method is called by JinroGame.checkNameDuplication(String str).
	def checkNameDuplication(name) {
		return !actors.find { it.name == name };
	}
	// Returns the proper name for the Skype ID.
	// This method is called by JinroGame.getNamebySkypeid(String skypeid).
	def getNamebySkypeid(skypeid) {
		return actors.find { it.skypeid == skypeid }?.name;
	}
	// Deals the role to each of the actors.
	// This method is called by JinroGame.dealRoles().
	def dealRoles() {
	}
	// Add to the dead's history.
	// This method is called by JinroGame.addHistory(Object actor).
	def addHistory(actor) {
	}
	// Notifies the each of actors of his/her role.
	// This method is called by JinroInterface.makeDialogStartGame().
	def noticeRoles() {
	}
	// Sends the command for voting or any actions.
	// This method is called by JinroGame.sendMessage(String name, String type, String target).
	def sendMessage(name, type, target) {
	}
}
